<?php

/*
 * 2007-2014 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author PrestaShop SA <contact@prestashop.com>
 *  @copyright  2007-2014 PrestaShop SA
 *  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class TntAccountAddress extends ObjectModel {

    /** @var integer Customer id which address belongs to */
    public $id_tnt_account;

    /** @var integer Country id */
    public $id_country;

    /** @var integer State id */
    public $id_state;

    /** @var string Country name */
    public $country;

    /** @var string Address first line */
    public $address_1;
    public $address_2;
    public $address_3;

    /** @var string Postal code */
    public $postcode;

    /** @var string City */
    public $city;
    protected static $_idZones = array();
    protected static $_idCountries = array();

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'tnt_account_address',
        'primary' => 'id_tnt_account_address',
        'fields' => array(
            'id_tnt_account' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => true),
            'id_country' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedId', 'required' => true),
            'id_state' => array('type' => self::TYPE_INT, 'validate' => 'isNullOrUnsignedId'),
            'address_1' => array('type' => self::TYPE_STRING, 'validate' => 'isAddress', 'required' => true, 'size' => 30),
            'address_2' => array('type' => self::TYPE_STRING, 'validate' => 'isAddress', 'required' => false, 'size' => 30),
            'address_3' => array('type' => self::TYPE_STRING, 'validate' => 'isAddress', 'required' => false, 'size' => 30),
            'postcode' => array('type' => self::TYPE_STRING, 'validate' => 'isPostCode', 'size' => 12),
            'city' => array('type' => self::TYPE_STRING, 'validate' => 'isCityName', 'required' => true, 'size' => 64),
        ),
    );
    protected $_includeVars = array('addressType' => 'table');
    protected $_includeContainer = false;

    /**
     * Build an address
     *
     * @param integer $id_address Existing address id in order to load object (optional)
     */
    public function __construct($id_address = null, $id_lang = null) {
        parent::__construct($id_address);

        /* Get and cache address country name */
        if ($this->id)
            $this->country = Country::getNameById($id_lang ? $id_lang : Configuration::get('PS_LANG_DEFAULT'), $this->id_country);
    }

    /**
     * @see ObjectModel::add()
     */
    public function add($autodate = true, $null_values = false) {
        if (!parent::add($autodate, $null_values))
            return false;

        if (Validate::isUnsignedId($this->id_tnt_account))
            Customer::resetAddressCache($this->id_tnt_account);
        return true;
    }

    public function update($null_values = false) {
        // Empty related caches
        if (isset(self::$_idCountries[$this->id]))
            unset(self::$_idCountries[$this->id]);
        if (isset(self::$_idZones[$this->id]))
            unset(self::$_idZones[$this->id]);

        if (Validate::isUnsignedId($this->id_tnt_account))
            Customer::resetAddressCache($this->id_tnt_account);

        return parent::update($null_values);
    }

    /**
     * Returns fields required for an address in an array hash
     * @return array hash values
     */
    public static function getFieldsValidate() {
        $tmp_addr = new Address();
        $out = $tmp_addr->fieldsValidate;

        unset($tmp_addr);
        return $out;
    }

    /**
     * @see ObjectModel::validateController()
     */
    public function validateController($htmlentities = true) {
        $errors = parent::validateController($htmlentities);
        return $errors;
    }

    /**
     * Get zone id for a given address
     *
     * @param integer $id_address Address id for which we want to get zone id
     * @return integer Zone id
     */
    public static function getZoneById($id_address) {
        if (!isset($id_address) || empty($id_address))
            return false;
        if (isset(self::$_idZones[$id_address]))
            return self::$_idZones[$id_address];

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
		SELECT s.`id_zone` AS id_zone_state, c.`id_zone`
		FROM `' . _DB_PREFIX_ . 'tnt_account_address` a
		LEFT JOIN `' . _DB_PREFIX_ . 'country` c ON c.`id_country` = a.`id_country`
		LEFT JOIN `' . _DB_PREFIX_ . 'state` s ON s.`id_state` = a.`id_state`
		WHERE a.`id_tnt_account_address` = ' . (int) $id_address);

        self::$_idZones[$id_address] = (int) ((int) $result['id_zone_state'] ? $result['id_zone_state'] : $result['id_zone']);
        return self::$_idZones[$id_address];
    }

    /**
     * Check if country is active for a given address
     *
     * @param integer $id_address Address id for which we want to get country status
     * @return integer Country status
     */
    public static function isCountryActiveById($id_address) {
        if (!isset($id_address) || empty($id_address))
            return false;

        $cache_id = 'Address::isCountryActiveById_' . (int) $id_address;
        if (!Cache::isStored($cache_id)) {
            $result = (bool) Db::getInstance(_PS_USE_SQL_SLAVE_)->getvalue('
			SELECT c.`active`
			FROM `' . _DB_PREFIX_ . 'tnt_account_address` a
			LEFT JOIN `' . _DB_PREFIX_ . 'country` c ON c.`id_country` = a.`id_country`
			WHERE a.`id_tnt_account_address` = ' . (int) $id_address);
            Cache::store($cache_id, $result);
        }
        return Cache::retrieve($cache_id);
    }

    public static function getCountryAndState($id_address) {
        if (isset(self::$_idCountries[$id_address]))
            return self::$_idCountries[$id_address];
        if ($id_address)
            $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
			SELECT `id_country`, `id_state`, `postcode` FROM `' . _DB_PREFIX_ . 'tnt_account_address`
			WHERE `id_tnt_account_address` = ' . (int) $id_address);
        else
            $result = false;
        self::$_idCountries[$id_address] = $result;
        return $result;
    }

    public static function getAddresById($id_tnt_account_address) {



        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS('
		SELECT tnt_acc.*, ctr.iso_code as company_country, sta.name as company_province, account.name as account_name, account.loginuser, account.password
		FROM `' . _DB_PREFIX_ . 'tnt_account_address` tnt_acc
                left join `' . _DB_PREFIX_ . 'country` ctr on ctr.id_country = tnt_acc.id_country
                left join `' . _DB_PREFIX_ . 'state` sta on sta.id_country = ctr.id_country and sta.id_state = tnt_acc.id_state
                left join `' . _DB_PREFIX_ . 'tnt_account` account on account.id_tnt_account = tnt_acc.id_tnt_account 
		WHERE `id_tnt_account_address` = ' . (int) $id_tnt_account_address);

        foreach ($result as $row)
            $tntaccounts[$row['id_tnt_account_address']] = $row;

        return $tntaccounts;
    }

    /**
     * Specify if an address is already in base
     *
     * @param int $id_address Address id
     * @return boolean
     */
    public static function addressExists($id_address) {
        $key = 'address_exists_' . (int) $id_address;
        if (!Cache::isStored($key)) {
            $id_address = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('SELECT `id_tnt_account_address` FROM ' . _DB_PREFIX_ . 'tnt_account_address a WHERE a.`id_tnt_account_address` = ' . (int) $id_address);
            Cache::store($key, (bool) $id_address);
        }
        return Cache::retrieve($key);
    }

    /**
     * Initiliaze an address corresponding to the specified id address or if empty to the
     * default shop configuration
     *
     * @param int $id_address
     * @return Address address
     */
    public static function initialize($id_address = null) {
        // if an id_address has been specified retrieve the address
        if ($id_address) {
            $address = new Address((int) $id_address);

            if (!Validate::isLoadedObject($address))
                throw new PrestaShopException('Invalid address');
        }
        else {
            // set the default address
            $address = new Address();
            $address->id_country = (int) Context::getContext()->country->id;
            $address->id_state = 0;
            $address->postcode = 0;
        }

        return $address;
    }

    public static function getTntAddressForLabelForm($id_tnt_account) {
        $tntaccounts = array();
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS('
		SELECT ta.id_tnt_account_address as value,
                       CONCAT(ta.address_1,\' \',ta.address_2,\' \',ta.address_3,\' \', ta.city) as name
		FROM `' . _DB_PREFIX_ . 'tnt_account_address` ta 
		WHERE ta.deleted = 0 and ta.id_tnt_account = \'' . $id_tnt_account . '\'
		ORDER BY ta.city ASC');

        foreach ($result as $row)
            $tntaccounts[$row['value']] = $row;

        return $tntaccounts;
    }

    public static function getProductoByPedidoId($id_order) {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS(
                '
                    select
                    reco.*,
                    reco.total_weight as t_weight,
                    sta.name,
                    ctr.iso_code,
                    ord.id_order,
                    ord.reference,
                    ord.total_paid,
                    adr.firstname,
                    adr.lastname,
                    cus.email,
                    adr.address1,
                    adr.address2,
                    adr.postcode,
                    adr.city,
                    cus_mes.message,
                    adr.phone,
                    adr.phone_mobile,
                    ( SELECT SUM(product_weight * product_quantity) FROM ' . _DB_PREFIX_ . 'order_detail WHERE id_order = ord.id_order) as total_weight
                    FROM `' . _DB_PREFIX_ . 'orders` ord
                    left join `' . _DB_PREFIX_ . 'tnt_recogida_label` reco on reco.id_order = ord.id_order
                    inner join `' . _DB_PREFIX_ . 'customer` cus on  ord.id_customer = cus.id_customer
                    inner join `' . _DB_PREFIX_ . 'address` adr on ord.id_address_delivery = adr.id_address
                    left join `' . _DB_PREFIX_ . 'country` ctr on ctr.id_country = adr.id_country
                    left join `' . _DB_PREFIX_ . 'state` sta on sta.id_country = ctr.id_country and sta.id_state = adr.id_state
                    left join `' . _DB_PREFIX_ . 'customer_thread` cus_thr on cus_thr.id_order = ord.id_order
                    left join `' . _DB_PREFIX_ . 'customer_message` cus_mes on cus_mes.id_customer_thread = cus_thr.id_customer_thread 
                        

                        where ord.id_order = ' . $id_order . '
'
        );

        if ($result) {
            foreach ($result as $row) {
                $row['productos'] = self::getAllProductFromOrderId($row['id_order']);
                $tntaccounts = $row;
            }

            return $tntaccounts;
        }
        return false;
    }

    public static function getAllProductFromOrderId($id_order) {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS(
                '
                    select prod.*,ord.*, ord.product_weight, ord.id_order_detail, ord.product_name, ord.product_price, ord.total_price_tax_incl
                    FROM `' . _DB_PREFIX_ . 'order_detail` ord
                    inner join `' . _DB_PREFIX_ . 'product` prod on ord.product_id = prod.id_product
                        where ord.id_order = ' . $id_order . '
'
        );

        foreach ($result as $row) {
            $tntaccounts[$row['id_order_detail']] = $row;
        }

        return $tntaccounts;
    }

    public static function saveTntOrderResult($id_order, $labels, $invoice, $consigment, $manifest) {



        $insertData = array(
            'id_order' => $id_order,
            'labels' => mysql_real_escape_string($labels),
            'invoice' => mysql_real_escape_string($invoice),
            'consigment' => mysql_real_escape_string($consigment),
            'manifest' => mysql_real_escape_string($manifest)
        );

        return Db::getInstance()->insert('tnt_order_result', $insertData);
    }

    public function getAddressIsUsed($id_tnt_account_address) {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS(
                '
                    select acc_add.id_tnt_account_address
                    FROM `' . _DB_PREFIX_ . 'tnt_recogida_label` acc_add
                        where acc_add.id_tnt_account_address = ' . $id_tnt_account_address . '
'
        );

        if (!$result) {
            return false;
        }
        return $result;
    }

}
